/* Encyklopedie jazyka C
 *
 * Kapitola Pole
 *
 */

/* Copyright (c) David Martinek, 2023
 * david.martinek.78 (zavináč) gmail.com
 *
 * This work is licensed under a Creative Commons
 * Attribution-NonCommercial-ShareAlike 4.0 International License.
 * (CC BY-NC-SA 4.0)
 * https://creativecommons.org/licenses/by-nc-sa/4.0/
 *
 * Toto dílo je licencováno pole licence Creative Commons
 * Uveďte původ-Neužívejte dílo komerčně-Zachovejte licenci 4.0 Mezinárodní
 * (CC BY-NC-SA 4.0)
 * https://creativecommons.org/licenses/by-nc-sa/4.0/deed.cs
 */

#define VERZE "4.3"
#define NAZEV "Pole"

// TODO Prostuduj si obsah tohoto hlavičkového souboru! Jsou tam zaklínadla,
// bez kterých některé věci ve Windows nefungují správně!
// - barevný výstup
// - printh - český tisk (překódovává z UTF8 na CP852)
// - formátovací a formátové značky pro printf/scanf podle normy
// - obsluha nabídek v kapitolách
#include "pomocny-modul.h"

// Implementace nabídkového menu
#include "menu.h"

#include <stdio.h>
#include <stdlib.h>
// #include <string.h>
#include <stdbool.h>
// #include <ctype.h>
// #include <math.h>
// #include <float.h>
// #include <limits.h>


// Hlavičky funkcí realizující jednotlivé kapitoly.
void typPole(void);
void jednorozmernaPole(void);
void jednorozmernyInicializator(void);
void jednorozmernyIndex(void);

void vicerozmernaPole(void);
void vicerozmernyInicializator(void);
void vicerozmernyIndex(void);

void ukazatelNaPole(void);
void ukazatelJakoPole(void);
void poleAPodprogramy1(void);
void poleAPodprogramy2(void);

void dynamickaPole(void);
void dynamickaPoleADT1D(void);
void dynamickaPoleADT2D(void);

void realokacePole(void);



int main(void)
{
  povolBarevnyVystup();
//  prepniRychlostVypraveni(2);

  Tmenu * menu = menuVytvor(
    NADPIS(
      "==============================================================\n"
      "Encyklopedie jazyka C - " NAZEV " (verze " VERZE ")\n"
      "(c) David Martinek, 2023\n"
      "Zároveň s programem sleduj i zdrojový text!\n"
      "==============================================================\n"
    )
    , zobrazMod
  );

  if (menu == NULL)
  {
    printg("Není dost paměti ani pro výrobu menu. Končím program.");
    return 0;
  }

  menuPridej(menu,   0, "\n" SEKCE("Pole obecně") , NULL);
  menuPridej(menu, 'A', "Datový typ pole\n", typPole);

  menuPridej(menu,   0, "\n" SEKCE("Jednorozměrná pole") , NULL);
  menuPridej(menu, 'B', "Deklarace a definice\n", jednorozmernaPole);
  menuPridej(menu, 'C', "Inicializace\n", jednorozmernyInicializator);
  menuPridej(menu, 'D', "Indexování a používání\n", jednorozmernyIndex);

  menuPridej(menu,   0, "\n" SEKCE("Vícerozměrná pole") , NULL);
  menuPridej(menu, 'E', "Deklarace a definice\n", vicerozmernaPole);
  menuPridej(menu, 'F', "Inicializace\n", vicerozmernyInicializator);
  menuPridej(menu, 'G', "Indexování a používání\n", vicerozmernyIndex);

  menuPridej(menu,   0, "\n" SEKCE("Pole a ukazatel") , NULL);
  menuPridej(menu, 'H', "Ukazatel na pole\n", ukazatelNaPole);
  menuPridej(menu, 'I', "Ukazatel jako pole\n", ukazatelJakoPole);
  menuPridej(menu, 'J', "Jednorozměrná pole a podprogramy\n", poleAPodprogramy1);
  menuPridej(menu, 'K', "Vícerozměrná pole a podprogramy\n", poleAPodprogramy2);

  menuPridej(menu,   0, "\n" SEKCE("Dynamicky alokovaná pole (pro pokročilé)") , NULL);
  menuPridej(menu, 'L', "Jednoduché pole\n", dynamickaPole);
  menuPridej(menu, 'M', "ADT 1D pole\n", dynamickaPoleADT1D);
  menuPridej(menu, 'N', "ADT 2D pole\n", dynamickaPoleADT2D);
  menuPridej(menu, 'O', "Realokace pole\n", realokacePole);

  menuPridej(menu,   0, "\n" SEKCE("Nastavení programu"), NULL);
  menuPridej(menu, '0', "Nastav vyprávěcí mód\n", nastavRychlostVypraveni);

  while (menuVolba(menu) != 'Q') {}

  printg("Končím program.\n");
  menuZrus(menu);
  return 0;
}



////////////////////////////////////////////////////////////////////////////////
void typPole(void)
{
  zacatekKapitoly("Datový typ pole obecně");

  printh(
    SEKCE("Uživatelský typ")
    "- Programátor jej musí dotvořit při deklaraci proměnné.\n"
    "- Musí specifikovat jeho bázový typ, dimenze a rozměry.\n"
    "\n"
    SEKCE("Homogenní datový typ")
    "- Typ pole popisuje proměnnou složenou z menších proměnných - prvků.\n"
    "- Všechny prvky pole jsou proměnné stejného typu.\n"
    "\n"
    SEKCE("Bázový typ pole")
    "- Je to datový typ jednotlivých prvků pole.\n"
    "- Může to být jakýkoli datový typ (kromě typu " KK("void") ").\n"
    "  => Je-li bázovým typem pole také typ pole, jde o vícerozměrné pole.\n"
    "\n"
  );

  if (navratNeboSekce("Rozměry a dimenze")) return;
  if (pokracovat())
  {
    printh(
      "- Dimenze pole je počet rozměrů (jednorozměrné, dvourozměrné, ...).\n"
      "- Rozměr pole v dané dimenzi je počet prvků v tomto směru.\n"
      "- Jazyk C umí pracovat s vícerozměrnými poli.\n"
      "- Počet dimenzí a jejich rozměry zadává programátor při deklaraci pole.\n"
      "- Rozměry jednotlivých dimenzí pole se za běhu programu obvykle nemění.\n"
      "- Za běhu programu lze měnit pouze rozměr první dimenze zleva a pouze\n"
      "  u dynamických polí.\n"
      // Ale není to triviální. Viz Realokace pole.
      "- Za běhu programu nelze měnit počet dimenzí.\n"
      // Nedává to smysl a implementačně by to bylo obtížné.
      "\n"
      KOD(
      "  #define RADKU 5\n"
      "  #define Sloupcu 3\n"
      "  float matice[RADKU][SLOUPCU];\n"
      )
      "\n"
      "- Pole " KK("matice") " je má dvě dimenze, je to tedy dvourozměrné pole.\n"
      "- Konstanta " KK("RADKU") " udává rozměr v první, " KK("SLOUPCU") " ve druhé dimenzi.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Indexování pole")) return;
  if (pokracovat())
  {
    int prvek[5] = {4, -7, 2, 14, 1};
    printh(
      "- Prvky pole jsou přístupné pomocí svého pořadí, tzv. indexů.\n"
      "- Index prvku je celé číslo označující pořadí prvku v poli.\n"
      "- První prvek pole má pozici, tj. index, s hodnotou nula.\n"
      "\n"
      "- Prvky jsou všechny stejného typu a leží v paměti souvisle vedle sebe.\n"
      "- Například pole pěti celých čísel může v paměti vypadat takto:\n"
      "\n"
      "  index:     0    1    2    3    4\n"
      "          +----+----+----+----+----+\n"
      "  prvek   |  4 | -7 |  2 | 14 |  1 |\n"
      "          +----+----+----+----+----+\n"
      "\n"
      KOD(
      "  int prvek[5] = {4, -7, 2, 14, 1};\n"
      )
      "  => prvek[3]: %d\n"
      "\n"
      "- V poli " KK("prvek") " se na indexu 3 nachází hodnota %d.\n"
      "\n"
      , prvek[3]
      , prvek[3]
    // Poznámka
    // - Adresa každého prvku se vypočítá jako pole + index*sizeof(<bázový typ>),
    //   kde pole je adresa začátku pole.
    // - Přesně to dělá na pozadí operátor indexování pole[index].
    // - Přesně to dělá na pozadí adresová aritmetika: pole + index
    // - Výrazy pole[index] a *(pole+index) jsou ekvivalentní.
    // - Adresovou aritmetiku se ale nedoporučuje moc používat - není přehledná.

    // sizeof(<bázový typ>) je důvod, proč u vícerozměrných polí musí být předem
    // známy pevné rozměry vyšších dimenzí
    );
  }

  if (navratNeboSekce("Pole si v jazyce C nepamatuje svůj rozměr!")) return;
  if (pokracovat())
  {
    printh(
      "=> Musíme si jej pamatovat v samostatné proměnné.\n"
      "- Je to daň za efektivitu jazyka C (blíže viz zdrojový text).\n"
//      "- Jiné jazyky si pamatují délku pole v prostoru před jeho počátkem.\n"
//      "  C ale může za počátek považovat i prostřední prvek pole a nemá tedy\n"
//      "  kam takový údaj uložit.\n"
//      "  => V jazyce C jde do funkce předat místo pole ukazatel na jiný než\n"
//      "  počáteční prvek a funkce jej pak má za počátek.\n"
      "\n"
      "=> Pole nelze kopírovat pomocí přiřazení! Operátor " KK("=") " by nevěděl, kolik toho\n"
      "   má kopírovat. Musí se kopírovat v cyklu po jednotlivých prvcích nebo pomocí \n"
      "   knihovních funkcí " KK("memcpy") ", " KK("memmove") ", nebo " KK("strcpy") ", " KK("strncpy") " ze " KK("string.h") ".\n"
      "\n"
      "=> Pole nemůže být návratovým typem funkce!\n"
      "- Důvodem je nemožnost přiřadit výsledek takové funkce do jiné proměnné.\n"
      "- Dále viz část Pole a podprogramy.\n"
      "\n"
      "- Často je výhodné zabalit pole a jeho rozměry do typu struktura.\n"
      "  Většina zmiňovaných problémů se tak dá obejít. Viz kapitola Struktury.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Využití polí")) return;
  if (pokracovat())
  {
    printh(
      PODSEKCE("Textový řetězec (anglicky string)")
      "- Jazyk C používá pole znaků pro jejich uchovávání.\n"
      "- Řetězec končí znakem s nulovou hodnotou " KK("'\\0'") ".\n"
      "- Pole musí být alespoň o 1 znak delší než řetězec.\n"
      "- V delším poli jsou další prvky nevyužity. Mohou ale být využity později.\n"
      "\n"
      PODSEKCE("Číselné řady a posloupnosti")
      "- Bázový typ pole je některý z číselných typů.\n"
      "\n"
      PODSEKCE("Reprezentace tabulek")
      "- Bázový typ je struktura.\n"
      "- Každý prvek pak reprezentuje řádek tabulky.\n"
      "- Každá složka v prvku (struktuře) reprezentuje hodnotu ve sloupci.\n"
      "\n"
      PODSEKCE("Reprezentace matic, obrázků apod.")
      "- Používají se dvourozměrná pole (pole jednorozměrných polí).\n"
      "- Používají řádkový a sloupcový index (v tomto pořadí).\n"
      "\n"
    );
  }

  konecKapitoly(NULL);
}




////////////////////////////////////////////////////////////////////////////////
void jednorozmernaPole(void)
{
  zacatekKapitoly("Jednorozměrná pole");

  printh(
    "- Pole s jednou dimenzí.\n"
    "\n"
    SEKCE("Deklarace a definice proměnné")
    "\n"
    KOD(
    "  <bázový typ> jméno[rozměr];\n"
    )
    "\n"
    PODSEKCE("  <bázový typ>")
    "  - libovolný typ, kromě prázdného typu " KK("void") "\n"
    "\n"
    PODSEKCE("  rozměr")
    "  kladné celé číslo udávající počet prvků\n"
    "\n"
  );

  if (navratNeboSekce("Rozměr pole")) return;
  if (pokracovat())
  {
    printh(
      "- Zapisuje se v hranatých závorkách za identifikátorem proměnné.\n"
      "- Pozor! Hranaté závorky nejde při deklaraci vynechat. Aby proměnná\n"
      "  byla typu pole, nestačí ji pojmenovat \"pole\"...\n"
      "\n"
      KOD(
      "  int pole;     // tohle NENÍ pole\n"
      "  int cislo[3]; // tohle JE pole\n"
      )
      "\n"
      "- Rozměr může být zadán konstantou nebo proměnnou (od ISO C99).\n"
      "\n"
      KOD(
      "  int delka = 3;\n"
      "  double pole1[3];      // rozměr zadaný konstantou\n"
      "  double pole2[delka];  // rozměr zadaný výrazem s proměnnou\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Vynechaný rozměr")) return;
  if (pokracovat())
  {
    printh(
      "- Při deklaraci parametru podprogramu lze závorky nechat prázdné.\n"
      "- Počet prvků, které je třeba zpracovat, se zadává dalším parametrem.\n"
      "- Uvede-li se tento rozměr, bude ignorován.\n"
      "- Typ pole neuchovává informaci o své délce, proto je nutná další proměnná.\n"
      "\n"
      PODSEKCE("Příklad")
      "- Tyto hlavičky funkce jsou totožné.\n"
      "\n"
      KOD(
      "  void orejPole(float pole[], int delka);\n"
      "  void orejPole(float pole[10], int delka); // na hodnotě 10 nezáleží\n"
      "  void orejPole(float *pole, int delka);    // pole jako ukazatel na začátek\n"
      )
      "\n"
      PODSEKCE("Poznámky")
      "- U vícerozměrných polí se toto týká pouze poslední dimenze, rozměry ostatních\n"
      "  dimenzí vynechány být nesmí.\n"
      "- Při definici běžné proměnné je rozměr potřeba, aby překladač věděl, jak velké\n"
      "  pole má vyrobit.\n"
      "- Důvod, proč jde pole nahradit ukazatelem, viz část \"Pole a ukazatel\".\n"
      "\n"
    );
    /*
      Důvod, proč si pole nepamatuje svou délku, je třeba hledat ve způsobu, jak
      se v jazyce C pole předávají do podprogramů. Kvůli efektivitě se nikdy
      nepředávají hodnotou, protože by to vyžadovalo vytváření kopie při každém
      volání. Místo toho se předává vždy ukazatel na počátek pole.

      Jazyk C to má zařízeno tak, že v mnoha ohledech není mezi typem ukazatel
      na prvek a typem pole rozdíl - často jsou tyto typy zaměnitelné. Využívá
      se toho při dynamické alokaci a u parametrů funkcí. Důvodem pro toto chování
      je vysoká efektivita zpracování polí. Implementace polí v jiných jazycích je
      obvykle mnohem méně efektivní než v C.

      Tyto hlavičky funkcí jsou totožné:
      void orej(float pole[], int delka);
      void orej(float *pole, int delka);

      Zpátky k důvodu, proč si pole nepamatuje délku. V jiných jazycích se typicky
      délka pole ukládá těsně před první prvek pole. Ve skutečnosti to tak dělá
      například funkce malloc, když alokuje paměť (a dělá to nejen u polí).
      V jazyce C jde ale předat do funkce ukazatel na nějaký prostřední prvek
      a tato funkce pak má tento prvek za počáteční. Problém je, že před tímto
      prvkem žádný údaj o délce uložen není, proto je na programátorovi, aby si
      délku pamatoval odděleně. Zároveň mu to dává větší volnost - může po funkci
      chtít zpracovat i kratší úsek, než jaký pole zabírá. Je na něm, jakou délku
      funkci zadá.
    */
  }

  konecKapitoly(__func__);
}




////////////////////////////////////////////////////////////////////////////////
void jednorozmernyInicializator(void)
{
  zacatekKapitoly("Inicializace jednorozměrného pole");

  printh(
    "- Při definici pole s pevným rozměrem jde použít inicializátor.\n"
    "- Je to konstrukce, která umožňuje nastavit hodnoty prvků pole už při\n"
    "  výrobě pole v paměti.\n"
    "- Později už to jde pouze algoritmicky přiřazováním hodnot do jednotlivých\n"
    "  prvků.\n"
    "\n"
  );

  if (navratNeboSekce("Neinicializované pole")) return;
  if (pokracovat())
  {
// Vypínám toto varování překladače, protože to chci demonstrovat.
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
    float rada1[5]; // pole 5 desetinných čísel
    printh(
      "- Nejsou-li prvky inicializované, mohou obsahovat náhodné hodnoty.\n"
      "- Pozor! Při ladění mohou mít prvky hodnotu 0 a vytvářet tak mylný pocit,\n"
      "  že je vše v pořádku.\n"
      /*
        Ve skutečnosti v paměti zůstanou zbytky po předchozích výpočtech.
        Mám podezření, že některé debuggery paměť před startem programu vyčistí,
        možná aby v ní byly lépe vidět změny provedené programem. V některých
        situacích to je ale nešťastné chování, protože se pak hůře projevují chyby
        s neinicializovanými proměnnými.
      */
      "- Při použití neinicializovaného pole překladač hlásí varování.\n"
      // Pokud je zapnut přepínač -Wall nebo -Wmaybe-uninitialized.
      "\n"
      KOD(
      "  float rada1[3]; // pole 3 desetinných čísel\n"
      )
      "\n"
      "  => rada1[0]: %g\n"
      "  => rada1[1]: %g\n"
      "  => rada1[2]: %g\n"
      "\n"
      , rada1[0], rada1[1], rada1[2]
    );
#pragma GCC diagnostic pop
  }

  if (navratNeboSekce("Inicializátor")) return;
  if (pokracovat())
  {
    float rada2[3] = {1.2f, 2.3f, -3.4f};

    printh(
      "- Je to výčet prvků ve složených závorkách. Prvky se oddělují čárkami.\n"
      "- Jde použít pouze u polí s konstantním rozměrem.\n"
      "- U polí s rozměrem zadaným pomocí proměnné nelze použít!\n"
      "\n"
      KOD(
      "  float rada2[3] = {1.2, 2.3, -3.4};\n"
      )
      "\n"
      "  => rada2[0]: %g, rada2[1]: %g, rada2[2]: %g\n"
      "\n"
      ,rada2[0], rada2[1], rada2[2]
    );

    printh(
      "- Pozor! Inicializátor se dá použít POUZE PŘI DEFINICI PROMĚNNÉ.\n"
      "  V jazyce C nejde do proměnné typu pole později nic přiřadit!\n"
      "  Pak jde přiřazovat jen do samostatných prvků.\n"
    );
    /*
      Důvod, proč pole jako celek nejde přiřazovat, souvisí s tím, že si pole
      nepamatuje svou délku. Překladač prostě obecně nemá odkud vzít informaci
      o tom, kolik prvků by měl kopírovat.

      Místo kopírování přiřazením se dají použít systémové funkce memcpy a memmove.
    */

    float rada3[3];
  //  Odkomentuj to a vyzkoušej
  //  rada3 = {1.2, 2.3, -3.4};  // Tohle je syntaktická chyba!
    rada3[0] = 1.2; rada3[1] = 2.3; rada3[2] = -3.4; // tohle je ok

    printh(
      "\n"
      KOD(
      "  float rada3[3];\n"
      "  //rada3 = {1.2, 2.3, -3.4};  // Tohle je syntaktická chyba!\n"
      "  rada3[0] = 1.2; rada3[1] = 2.3; rada3[2] = -3.4; // tohle je ok \n"
      )
      "\n"
      "  => rada3: %4.1f, %4.1f, %4.1f\n"
      "\n"
      , rada3[0], rada3[1], rada3[2]
    );
  }

  if (navratNeboSekce("Inicializátor s vynechanými prvky")) return;
  if (pokracovat())
  {
    float rada4[4] = {5.0, -2.5};
    float rada5[4] = {2.0};

    printh(
      "- Pokud je v inicializátoru inicializováno méně prvků, než je rozměr pole,\n"
      "  budou nevyjmenované prvky nastaveny vždy na nulu.\n"
      "- V tom případě musí být nastaven alespoň jeden prvek.\n"
      "- Rozměr pole musí být konstanta. (Nejde použít ani konstatní proměnnou.)\n"
      "\n"
      KOD(
      "  float rada4[4] = {5.0, -2.5};\n"
      "  float rada5[4] = {2.0};\n"
      )
      "\n"
      "  => rada4: %4.1f, %4.1f, %4.1f, %4.1f\n"
      "  => rada5: %4.1f, %4.1f, %4.1f, %4.1f\n"
      "\n"
      , rada4[0], rada4[1], rada4[2], rada4[3]
      , rada5[0], rada5[1], rada5[2], rada5[3]
    );
  }

  if (navratNeboSekce("Inicializátor s vynechaným rozměrem")) return;
  if (pokracovat())
  {
    float rada6[] = {1.5, 5.8}; // délka je 2

    printh(
      "- Pokud je použit inicializátor, jde vynechat rozměr pole.\n"
      "- Rozměr se pak dopočítá z inicializátoru.\n"
      "\n"
      KOD(
      "  float rada6[] = {1.5, 5.8}; // délka je 2\n"
      )
      "\n"
      "  => rada6: %4.1f, %4.1f\n"
      "\n"
      , rada6[0], rada6[1]
    );
  }

  if (navratNeboSekce("Inicializace konkrétních prvků")) return;
  if (pokracovat())
  {
    float rada7[5] = {2.4, [3] = 7.1};

    printh(
      "- Funguje od ISO C99.\n"
      "- V inicializátoru jde přiřadit hodnotu prvku na konkrétním indexu.\n"
      "- Nevyjmenované prvky jsou opět nulové.\n"
      "\n"
      KOD(
      "  float rada7[5] = {2.4, [3] = 7.1};\n"
      )
      "\n"
      "  => rada7: %4.1f, %4.1f, %4.1f, %4.1f, %4.1f\n"
      "\n"
      , rada7[0], rada7[1], rada7[2], rada7[3], rada7[4]
    );
  }

  konecKapitoly(__func__);
}




////////////////////////////////////////////////////////////////////////////////
void jednorozmernyIndex(void)
{
  zacatekKapitoly("Indexování jednorozměrného pole");

  printh(
    "- Slouží pro práci s jednotlivými prvky vytvořeného pole.\n"
    "- Pro realizaci indexování slouží operátor indexování " KK("[]") ".\n"
    "- Operátor indexování vybírá právě jeden prvek pole.\n"
    "\n"
  );

  if (navratNeboSekce("Prvek pole")) return;
  if (pokracovat())
  {
    printh(
      "- Každý prvek pole je samostatná proměnná.\n"
      "- Je to L-hodnota, takže z něj jde číst a jde do něj zapsat.\n"
      "- Datovým typem prvku pole je bázový typ pole.\n"
      "- Prvky pole jsou v paměti umístěny souvisle za sebou.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Syntaxe")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  pole[index]\n"
      )
      "\n"
      PODSEKCE("pole")
      "- Je to proměnná typu jednorozměrné pole,\n"
      "- nebo proměnná typu ukazatel na bázový typ, typicky když jde o pole\n"
      "  vytvářené pomocí funkce " KK("malloc") ".\n"
      "\n"
      PODSEKCE("index")
      "- Je to celočíselný výraz udávající pořadí prvku v poli.\n"
      "- První prvek pole leží vždy na indexu s hodnotou 0.\n"
      "- Poslední prvek pole leží vždy na indexu s hodnotou délka-1.\n"
      "\n"
      PODSEKCE("operátor indexování []")
      "- Je to operátor, tudíž musí mít operandy: pole a index.\n"
      "- Vrací prvek pole, tedy L-hodnotu.\n"
      "- Pozor! Při deklaraci a definici proměnné nejsou hranaté závorky tento\n"
      "  operátor. Pak jde o součást názvu datového typu!\n"
      "\n"
    );
  }

  if (navratNeboSekce("Ukázka: průchod polem od začátku do konce")) return;
  if (pokracovat())
  {
    #define DELKA 10
    int cislo[DELKA]; // definice pole s délkou danou pomocí konstanty

    for (int i = 0; i < DELKA; ++i)
    {
      cislo[i] = i; // uloží do prvků hodnotu jejich indexů
    }

    printh(
      "- První prvek leží na indexu 0.\n"
      "- Podmínka cyklu je " KK("i < DELKA") ".\n"
      "- Indexování hodnotou " KK("i=DELKA") " by vedlo za hranici pole. Poslední prvek\n"
      "  leží na pozici indexu " KK("DELKA-1") ".\n"
      "\n"
      KOD(
      "  #define DELKA 10\n"
      "  int cislo[DELKA]; // definice pole s délkou danou pomocí konstanty\n"
      "\n"
      "  // inicializace prvků pole\n"
      "  for (int i = 0; i < DELKA; ++i)\n"
      "  {\n"
      "    cislo[i] = i; // uloží do prvků hodnotu jejich indexů\n"
      "  }\n"
      "\n"
      "  // tisk prvků pole\n"
      "  printf(\"cislo: %%d\", cislo[0]);\n"
      "  for (int i = 1; i < DELKA; ++i)\n"
      "  {\n"
      "    printf(\", %%d\", cislo[i]);\n"
      "  }\n"
      )
      "\n"
      "  => "
    );

    // tisk prvků pole
    printf("cislo: %d", cislo[0]);
    for (int i = 1; i < DELKA; ++i)
    {
      printf(", %d", cislo[i]);
    }

    printh("\n");
  }

  konecKapitoly(__func__);
}




////////////////////////////////////////////////////////////////////////////////
void vicerozmernaPole(void)
{
  zacatekKapitoly("Vícerozměrná pole");

  printh(
    "- Vícerozměrná pole mají více než jednu dimenzi.\n"
    "- Dvourozměrné pole je pole polí,\n"
    "  => tedy jednorozměrné pole řádků - jednorozměrných polí.\n"
    "- Při 1D pohledu je bázovým typem tohoto pole jiné pole.\n"
    "- Obvykle za bázový typ považujeme typ prvku v poslední dimenzi.\n"
    "\n"
    "- Vícerozměrná pole se konstruují stejně - například trojrozměrné pole\n"
    "  je ve skutečnosti jednorozměrné pole, jehož prvky jsou dvourozměrná pole.\n"
    "\n"
  );

  if (navratNeboSekce("Deklarace a definice")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  <bázový typ> jméno[dimenze1][dimenze2]\n"
      "  <bázový typ> jméno[dimenze1][dimenze2][dimenze3][atd.]\n"
      )
      "\n"
      PODSEKCE("  <bázový typ>")
      "  - typ prvku v poslední dimenzi (nejpravější)\n"
      "\n"
      PODSEKCE("  jméno")
      "  - identifikátor proměnné typu vícerozměrné pole\n"
      "\n"
      PODSEKCE("  dimenze1, dimenze2, ...")
      "  - rozměry pole v jednotlivých dimenzích\n"
      "  - U dvourozměrných polí jsou dimenze v pořadí: řádky, sloupce\n"
      "\n"
    );
  }

  if (navratNeboSekce("Rozměry dimenzí")) return;
  if (pokracovat())
  {
//    int radku = 3, sloupcu = 5;
//    double m1[3][5];            // rozměr zadaný konstantou
//    double m2[radku][sloupcu];  // rozměr zadaný výrazem s proměnnou

    printh(
      "- Zapisují se v hranatých závorkách za identifikátorem proměnné.\n"
      "- Každá dimenze se zapisuje v samostatném páru závorek.\n"
      "\n"
      KOD(
      "  int matice[3][5]; // tohle je 2D pole o 3 řádcích a 5 sloupcích\n"
      )
      "\n"
      "- Rozměr může být zadán konstantami nebo proměnnými (od ISO C99),\n"
      "  což typicky bývají parametry podprogramu.\n"
      "\n"
      KOD(
      "  int radku = 3, sloupcu = 5;\n"
      "  double m1[3][5];            // rozměr zadaný konstantou\n"
      "  double m2[radku][sloupcu];  // rozměr zadaný výrazem s proměnnou\n"
      )
      "\n"
      "- Pozor! Pole s rozměry zadanými proměnnými se obtížně předávají do\n"
      "  podprogramů (je na to speciální syntaxe).\n"
      "- Zásobník má omezenou velikost. => Není dobré na něm vyrábět velká pole.\n"
      "- Reálně je lepší alokovat velká pole dynamicky na hromadě (heap).\n"
      "  Viz další kapitoly.\n"
  //    "  ...a zabalit jej spolu se skutečnými rozměry do struktury. Takto vyrobený\n"
  //    "  dynamický datový typ lze snadno používat jako parametry a dokonce\n"
  //    "  i jako návratový typ funkcí.\n"
      "\n"
    );

  }

  if (navratNeboSekce("Jak takové pole vypadá v paměti (na zásobníku)")) return;
  if (pokracovat())
  {
    int m3[2][3] = { {1, 2, 3}, {4, 5, 6} };
    int m4[6] = { 1, 2, 3, 4, 5, 6 };

    printh(
      "- Lokální vícerozměrné pole zabírá souvislou oblast zásobníku.\n"
      "- U 2D pole leží první prvek nového řádku hned za posledním prvkem\n"
      "  předchozího řádku.\n"
      "- Poznámka: U dynamicky alokovaných polí to vypadá jinak.\n"
      "\n"
      "- Tato dvě pole vypadají v paměti naprosto stejně.\n"
      "- Rozdíly v přístupu k nim si pamatuje překladač.\n"
      "\n"
      KOD(
      "  int m3[2][3] = { {1, 2, 3}, {4, 5, 6} };\n"
      "  int m4[6] = { 1, 2, 3, 4, 5, 6 };\n"
      )
      "\n"
      "  +---------+--------+--------+--------+--------+--------+--------+\n"
      "  |prvky    |m3[0][0]|m3[0][1]|m3[0][2]|m3[1][0]|m3[1][1]|m3[1][2]|\n"
      "  |hodnoty  |  %3d   |  %3d   |  %3d   |  %3d   |  %3d   |  %3d   |\n"
      "  +---------+--------+--------+--------+--------+--------+--------+\n"
      "\n"
      "  +---------+--------+--------+--------+--------+--------+--------+\n"
      "  |prvky    |  m4[0] |  m4[1] |  m4[2] |  m4[3] |  m4[4] |  m4[5] |\n"
      "  |hodnoty  |  %3d   |  %3d   |  %3d   |  %3d   |  %3d   |  %3d   |\n"
      "  +---------+--------+--------+--------+--------+--------+--------+\n"
      "\n"
      , m3[0][0], m3[0][1], m3[0][2], m3[1][0], m3[1][1], m3[1][2]
      , m4[0],    m4[1],    m4[2],    m4[3],    m4[4],    m4[5]
    );

    /* Poznámky
      Ačkoli tato dvě pole vypadají v paměti stejně, nejde o totožné typy.
      K oběma těmto typům si překladač pamatuje jiné údaje a jinak přistupuje
      k jednotlivým prvkům obou polí. V jazyce C tyto dvě proměnné NELZE vzájemně
      přetypovat.

      Někdy je potřeba mít jednorozměrné pole, ale pracovat s ním jako
      s dvourozměrným polem. Dá se použít tento trik:

      m4[r*POCET_SLOUPCU+s], což odpovídá m3[r][s],

      kde r je řádkový index, s je sloupcový index a POCET_SLOUPCU je délka řádku.

      Tento trik se dá použít například při dynamické alokaci, kdy z nějakého
      důvodu chci mít vícerozměrné pole souvislé (tj. alokované jako jeden blok,
      nikoli po jednotlivých samostatně alokovaných řádcích).

    */
  }

  konecKapitoly(__func__);
}




////////////////////////////////////////////////////////////////////////////////
void vicerozmernyInicializator(void)
{
  zacatekKapitoly("Inicializace vícerozměrného pole");

  printh(
    "- Inicializátor jde použít u polí s pevnými rozměry.\n"
    "- Je to konstrukce, která umožňuje nastavit hodnoty prvků pole už při\n"
    "  výrobě pole v paměti.\n"
    "- Později už to jde pouze algoritmicky přiřazováním hodnot do jednotlivých\n"
    "  prvků.\n"
    "\n"
  );

  if (navratNeboSekce("Inicializátor")) return;
  if (pokracovat())
  {
    float matice[3][4] = {
      {  1.2,  2.3, -3.4,  4.5},  // řádky jsou odděleny čárkami
      {   .3, 2e-2,  1.4, -3.5},
      {-2.74,  2.3, -3.4,  6.0},  // čárka za posledním řádkem nevadí
    };

    printh(
      "- Jde o stejnou konstrukci jako u 1D polí.\n"
      "- Vícerozměrné pole je pole polí => inicializátory jsou vnořené.\n"
      "- Pole musí mít konstantní rozměry.\n"
      "- U polí s rozměry zadanými pomocí proměnných nelze použít!\n"
      "\n"
      "- Pro lepší přehlednost je dobré inicializátory vhodně formátovat.\n"
      "\n"
      KOD(
      "  float matice[3][4] = {\n"
      "    { %5.1f, %5.1f, %5.1f, %5.1f},  // řádky jsou odděleny čárkami\n"
      "    { %5.1f, %5.1f, %5.1f, %5.1f},\n"
      "    { %5.1f, %5.1f, %5.1f, %5.1f},  // čárka za posledním řádkem nevadí\n"
      "  };\n"
      )
      "\n"
      , matice[0][0], matice[0][1], matice[0][2], matice[0][3]
      , matice[1][0], matice[1][1], matice[1][2], matice[1][3]
      , matice[2][0], matice[2][1], matice[2][2], matice[2][3]
    );

    printh(
      "- Pozor! Inicializátor se dá použít POUZE PŘI DEFINICI PROMĚNNÉ.\n"
      "  V jazyce C nejde do proměnné typu pole později nic přiřadit!\n"
      "  Pak jde přiřazovat jen do samostatných prvků.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Inicializace vybraných prvků")) return;
  if (pokracovat())
  {
// Vypínám toto varování, protože to je hack, který dělám záměrně.
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wunused-variable"
    { // Tohle je vnořený blok určený k "zašpinění" vrcholu zásobníku.
      float XXX[4][4] = {
        {   .3, 2e-2,  1.4, -3.5},
        {   .3, 2e-2,  1.4, -3.5},
        {   .3, 2e-2,  1.4, -3.5},
        {   .3, 2e-2,  1.4, -3.5},
      };
      // Tady blok končí, proměnná XXX zaniká, ale na zásobníku zůstanou nenulové
      // paměťové buňky. Na tomto místě zásobníku vzápětí vznikne proměnná
      // diagonala. Pokud by nebyla inicializována, její prvky budou mít hodnoty,
      // které v paměti zůstaly po proměnné XXX.
      // Takto snadno ověříme, že při použití inicializátoru skutečně
      // nevyjmenované prvky budou nastaveny na hodnotu nula.
    }
#pragma GCC diagnostic pop

    float diagonala[4][4] = {
      [3] = {[3]=4.0},
      [1] = {[1]=2.0},
      [2] = {[2]=3.0},
      [0] = {[0]=1.0},
    };

    printh(
      "- Stále platí, že při použití inicializátoru se nevyjmenované prvky\n"
      "  nastaví na nulu. (Bez použití inicializátoru jsou neinicializované.)\n"
      "- Lze určit konkrétní prvky, které se inicializují.\n"
      "- V tom případě mohou být zapsány v libovolném pořadí.\n"
      "\n"
      KOD(
      "  float diagonala[4][4] = {\n"
      "    [3] = {[3]=4.0},\n"
      "    [1] = {[1]=2.0},\n"
      "    [2] = {[2]=3.0},\n"
      "    [0] = {[0]=1.0},\n"
      "  };\n"
      )
      "\n"
      "  diagonala: | %3.1f %3.1f %3.1f %3.1f |\n"
      "             | %3.1f %3.1f %3.1f %3.1f |\n"
      "             | %3.1f %3.1f %3.1f %3.1f |\n"
      "             | %3.1f %3.1f %3.1f %3.1f |\n"
      "\n"
      , diagonala[0][0], diagonala[0][1], diagonala[0][2], diagonala[0][3]
      , diagonala[1][0], diagonala[1][1], diagonala[1][2], diagonala[1][3]
      , diagonala[2][0], diagonala[2][1], diagonala[2][2], diagonala[2][3]
      , diagonala[3][0], diagonala[3][1], diagonala[3][2], diagonala[3][3]
    );
  }

  konecKapitoly(__func__);
}




////////////////////////////////////////////////////////////////////////////////
void vicerozmernyIndex(void)
{
  zacatekKapitoly("Indexování vícerozměrných polí");

  printh(
    "- Prvky v poslední dimenzi jsou proměnné, tj. l-hodnoty.\n"
    "- Tyto prvky jsou stejného typu, jako je bázový typ pole.\n"
    "\n"
    "- Indexuje se postupným aplikováním operátoru " KK("[]") " v pořadí od nejvyšší\n"
    "  dimenze po nejnižší.\n"
    "- Indexy dimenzí mají stejné pořadí jako při deklaraci pole.\n"
    "- U 2D pole se nejdříve indexují řádky, poté sloupce.\n"
    "\n"
    "- Pokud se 2D pole indexuje jen 1D indexem, je výsledný prvek 1D pole - řádek.\n"
    "\n"
  );

  if (navratNeboSekce("Vzor pro průchod 2D polem")) return;
  if (pokracovat())
  {
    #define RADKU 3
    #define SLOUPCU 5
    int m[RADKU][SLOUPCU];

    for (int r = 0; r < RADKU; ++r)
    {
      for (int s = 0; s < SLOUPCU; ++s)
      {
        m[r][s] = r * SLOUPCU + s;
      }
    }

    printh(
      "- Řádkový a sloupcový index je dobré pojmenovat výstižně (např. r, s).\n"
      "  Indexy i, j se snadno zamění. Indexy x, y spolehlivě zmatou i zkušené\n"
      "  programátory (řádkový index by totiž byl y, sloupcový x).\n"
      /* Poznámka
      Důvod, proč se 2d pole indexuje v pořadí pole[y][x] je v tom, že většina
      algoritmů se po maticích pohybuje po řádcích (tj. nejprve zpracují prvky
      v jednom řádku, pak v dalším atd.). Průchod v tomto pořadí je
      nejefektivnější, protože se po paměti pohybujeme kontinuálně. Průchod po
      sloupcích je sice také možný, ale vyžaduje neustálé přeskakování
      na vzdálené prvky. U polí malých rozměrů to asi nevadí, ale u velkých
      matic už může být rozdíl v rychlosti citelný.
      */
      "\n"
      KOD(
      "  #define RADKU 3\n"
      "  #define SLOUPCU 5\n"
      "  int m[RADKU][SLOUPCU];\n"
      "  \n"
      "  for (int r = 0; r < RADKU; ++r) \n"
      "  {\n"
      "    for (int s = 0; s < SLOUPCU; ++s)\n"
      "    {\n"
      "      m[r][s] = r * SLOUPCU + s;\n"
      "    } \n"
      "  }\n"
      )
      "\n"
      "  => m: "
    );

    for (int r = 0; r < RADKU; ++r)
    {
      printf("|");
      for (int s = 0; s < SLOUPCU; ++s)
      {
        printf(" %2d", m[r][s]);
      }
      printf(" |\n        ");
    }

    printh(
      "\n"
    );
  }

  if (navratNeboSekce("2D zacházení s 1D polem")) return;
  if (pokracovat())
  {
    int pole[RADKU*SLOUPCU];
    for (int r = 0; r < RADKU; ++r)
    {
      for (int s = 0; s < SLOUPCU; ++s)
      {
        pole[r*SLOUPCU+s] = r * SLOUPCU + s;
      }
    }

    printh(
      "- 1D pole nejde přetypovat na 2D pole.\n"
      // Jde to například pomocí typu union, ale rozhodně to není běžná konstrukce.
      "- Jde ale použít 2 indexy a při indexování je přepočítat na 1D index\n"
      "  podle vztahu:\n"
      "\n"
      "  Index1D = IndexRadku*POCET_SLOUPCU + IndexSloupce\n"
      "\n"
      "  Počet sloupců 2D pole odpovídá délce každého řádku.\n"
      "\n"
      KOD(
      "  int pole[RADKU*SLOUPCU];\n"
      "  for (int r = 0; r < RADKU; ++r)\n"
      "  {\n"
      "    for (int s = 0; s < SLOUPCU; ++s)\n"
      "    {\n"
      "      pole[r*SLOUPCU+s] = r * SLOUPCU + s;\n"
      "    }\n"
      "  }\n"
      )
      "\n"
      "  pole => |"
    );

    for (int r = 0; r < RADKU; ++r)
    {
      for (int s = 0; s < SLOUPCU; ++s)
      {
        printf(" %2d", pole[r*SLOUPCU+s]);
      }
    }
    printf(" |\n\n");
  }

  konecKapitoly(__func__);
}




////////////////////////////////////////////////////////////////////////////////
void ukazatelNaPole(void)
{
  zacatekKapitoly("Ukazatel na pole");

  printh(
    "- Proměnná typu pole obsahuje adresu začátku pole.\n"
    "- Typ pole ale není totožný s typem ukazatel. Překladač si o poli pamatuje\n"
    "  více údajů než o ukazateli.\n"
    "- Typ pole je kompatibilní s typem ukazatel na bázový typ.\n"
    "\n"
    /* Přesněji řečeno je typ pole kompatibilní s typem ukazatel na bázový typ.
    Proměnná typu pole obsahuje adresu počátku pole. Při indexování se pak
    pozice jednotlivých prvků spočítá jako adresa začátku pole +
    + index*sizeof(bázový typ).

    Na rozdíl od proměnné typu ukazatel nejde proměnnou typu pole měnit, protože
    bychom přišli o údaj o tom, kde leží začátek pole. V podstatě lze proměnnou
    proměnnou typu pole považovat za konstantní ukazatel na začátek pole.

    Ačkoli obecně platí, že pole si v jazyce C nepamatují svou délku,
    u proměnných typu pole si tuto délku může pamatovat překladač nebo si ji
    může ukládat do paměti třeba před samotné pole. Proto například operátor
    sizeof(pole) dokáže vrátit velikost pole (v bajtech). Musí ale jít skutečně
    o proměnnou typu pole a ne o ukazatel. V tom je rozdíl mezi typem pole
    a typem ukazatel.

    Poznámka: Tvrzení, že pole si v C nepamatují svou délku není s předchozím
    odstavcem v rozporu. Tu délku si pamatuje překladač (alespoň u polí
    s konstantní délkou) a musí jít skutečně o proměnnou typu pole. Překladač
    nedokáže spočítat délku, když jde o proměnnou typu ukazatel na začátek pole,
    ani když jde o parametr podprogramu, protože za něj jde dosadit jak pole
    tak ukazatel, nebo ukazatel doprostřed pole atd.

    */
  );

  if (navratNeboSekce("Získání ukazatele na pole")) return;
  if (pokracovat())
  {
    printh(
      "- Použitím jména proměnné bez operátoru indexování.\n"
      "- Není potřeba používat referenční operátor " KK("&") ".\n"
      "\n"
      KOD(
      "  char slovo[5];\n"
      "  char * ukazatelNaPole = slovo;\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Předávání pole jako argumentu podprogramu")) return;
  if (pokracovat())
  {
    printh(
      "- Pole se nikdy nepředává hodnotou -- nikdy se celé nekopíruje na zásobník.\n"
      "- Předává se vždy adresa začátku pole.\n"
      "=> Při volání podprogramu se pole nepředává s operátorem " KK("&") ".\n"
      "=> Podprogram může měnit prvky předávaného pole.\n"
      "   Podprogram se může zavázat, že pole měnit nebude tím, že se v hlavičce\n"
      "   před deklaraci parametru typu pole dá klíčové slovo " KK("const") ".\n"
      "\n"
      "- Typický příklad, kdy se při předávání odkazem před argument nedává " KK("&") ":\n"
      "\n"
      KOD(
      "  char jmeno[15+1];\n"
      "  printf(\"Zadej sve jmeno: \");\n"
      "  scanf(\"%%15s\", jmeno);  // tedy ne &jmeno\n"
      )
      // V tomto případě by operátor & nevadil, protože by stejně vrátil adresu
      // začátku pole. Jeho použití je ale nadbytečné.
      // Nebezpečné by to začalo být, když by proměnná jmeno byla deklarována
      // jako ukazatel:
      // char *jmeno;
      // Pak by ji bylo nutné nějak nastavit, aby ukazovala na skutečné pole:
      // jmeno = malloc(16*sizeof(char));
      // V tomto případě by použití & při volání scanf mohlo způsobit havárii:
      // scanf("%15s", &jmeno);
      // Výraz &jmeno vrací adresu proměnné jmeno, ne adresu začátku pole - ta
      // je uložena právě v proměnné jmeno...
      // Ještě horší je, že by to pravděpodobně nezpůsobilo havárii hned. Došlo
      // by k přepsání proměnných uložených okolo proměnné jmeno a k havárii
      // by zřejmě došlo v důsledku toho, ale na úplně jiném místě programu.
      "\n"
    );

    char jmeno[15+1];
    printf("Zadej sve jmeno: ");
    int e = scanf("%15s", jmeno);

    if (e == 1)
      printf("  => jmeno: %s\n", jmeno);
    else
      printg("  => Chyba! String se nepovedlo načíst.\n");

    printh(
      "\n"
      "- U jiných typů bude & potřeba:\n"
      "\n"
      KOD(
      "  int vek;\n"
      "  printf(\"Zadej svuj vek: \");\n"
      "  scanf(\"%%d\", &vek);  // tady & být musí\n"
      )
      "\n"
    );

    int vek;
    printf("Zadej svuj vek: ");
    e = scanf("%d", &vek);
    pozerKonecRadku();

    if (e == 1)
      printf("  => vek: %d\n", vek);
    else
      printg("  => Chyba! Hodnotu se nepovedlo načíst\n");

    printh("\n");
  }

  konecKapitoly(__func__);
}



////////////////////////////////////////////////////////////////////////////////
void ukazatelJakoPole(void)
{
  float poleCisel[] = {1.2, 3.4, 5.6};
  float * uPole = poleCisel;

  zacatekKapitoly("Ukazatel jako pole");

  printh(
    "- Do proměnné typu ukazatel jde přiřadit proměnnou typu pole.\n"
    "  Přiřadí se pak adresa začátku pole, resp. prvního prvku pole.\n"
    "- Opačně to nejde. Proměnnou typu pole měnit nejde (jen jeho prvky).\n"
    "\n"
    "- Typické použití: dynamicky alokované pole, parametr funkce\n"
    "\n"
    KOD(
    "  float poleCisel[] = {1.2, 3.4, 5.6};\n"
    "  float * uPole = poleCisel;\n"
    )
    "\n"
    "  => poleCisel: |%g %g %g|\n"
    "  => adresa uložená v uPole: %p\n"
    "  => adresa reprezentovaná proměnnou poleCisel: %p\n"
    "  => adresa prvního prvku pole &poleCisel[0]: %p\n"
    "\n"
    , poleCisel[0], poleCisel[1], poleCisel[2]
    , (void *)uPole        // přetypování na beztypový ukazatel kvůli značce %p
    , (void *)poleCisel
    , (void *)&poleCisel[0]
  );


  float * uDynPole = malloc(3*sizeof(float));
  if (uDynPole == NULL)
  {
    printf("Pole se nepovedlo alokovat!\n");
    return;
  }

  if (navratNeboSekce("Ukazatel na dynamicky alokované pole")) return;
  if (pokracovat())
  {
    printh(
      "- Typické využití příbuznosti typu ukazatel a typu pole.\n"
      "- Pole se alokuje dynamicky na hromadě (heap), ale pracuje se s ním\n"
      "  pomocí ukazatele na jeho začátek.\n"
      "\n"
      "- Pozor! Při dynamické alokaci nedojde k inicializaci prvků.\n"
      "- Nejde zde použít inicializátor.\n"
      // To jde jen u lokálních proměnných.
      "- Inicializace se musí udělat ručně.\n"
      "- Po dymamické alokaci nezapomínej na testování!\n"
      "- Detailněji viz další sekce.\n"
      "\n"
      KOD(
      "  float * uDynPole = malloc(3*sizeof(float));\n"
      "  if (uDynPole == NULL)\n"
      "  {\n"
      "    printf(\"Pole se nepovedlo alokovat!\\n\");\n"
      "    return;\n"
      "  }\n"
      )
      "\n"
      "  => uDynPole: |%g %g %g|\n"
      "  => adresa uložená v uDynPole: %p\n"
      "  => adresa prvního prvku pole &uDynPole[0]: %p\n"
      "\n"
      , uDynPole[0], uDynPole[1], uDynPole[2]
      , (void *)uDynPole     // přetypování na beztypový ukazatel kvůli značce %p
      , (void *)&uDynPole[0]
    );
  }

  uDynPole[1] = 9.9; // Změní druhý prvek uDynPole.
  uPole[1] = 8.8;    // Nepřímý odkaz na lokální pole - změní prvek poleCisel.

  if (navratNeboSekce("Indexování")) return;
  if (pokracovat())
  {
    printh(
      "- Na ukazatel jde aplikovat operátor indexování " KK("[]") ".\n"
      "\n"
      KOD(
      "  uDynPole[1] = 9.9; // Změní druhý prvek uDynPole.\n"
      "  uPole[1] = 8.8;    // Nepřímý odkaz na lokální pole - změní prvek poleCisel.\n"
      )
      "\n"
      "  => uDynPole: |%g %g %g|\n"
      "  => poleCisel: |%g %g %g|\n"
      "\n"
      , uDynPole[0], uDynPole[1], uDynPole[2]
      , poleCisel[0], poleCisel[1], poleCisel[2]
    );
  }

  free(uDynPole); // Nezapomeň uklidit!

  if (navratNeboSekce("Ukazatel na pole jako parametr funkce")) return;
  if (pokracovat())
  {
    printh(
      "- Díky příbuznosti typů ukazatel a pole jde parametr typu pole nahradit\n"
      "  parametrem typu ukazatel na bázový typ.\n"
      "\n"
      KOD(
      "  void zpracujPole(int * pole, int delka);\n"
      )
      "\n"
      "- Za první parametr jde dosadit jakkoli alokované jednorozměrné pole\n"
      "  celých čísel.\n"
      "- Více viz další sekci.\n"
      "\n"
    );
  }

  konecKapitoly(__func__);
}


////////////////////////////////////////////////////////////////////////////////
void poleAPodprogramy1(void)
{
  zacatekKapitoly("Jednodimenzionální pole a podprogramy");

  printh(
    "- Pole se zpracovávájí často pomocí podprogramů.\n"
    "- Pole se vždy předávají vnitřně odkazem, i když to syntakticky vypadá\n"
    "  jako předávání hodnotou. -> Proměnná typu pole ale v zásadě je ukazatel.\n"
    "\n"
  );

  if (navratNeboSekce("Pole jako parametr podprogramu")) return;
  if (pokracovat())
  {
    printh(
      "- V deklaraci parametru se vynechává (ignoruje) velikost dimenze.\n"
      "  Délku je potřeba zadat jiným parametrem.\n"
      "- Díky příbuznosti typů ukazatel a pole jde parametr typu pole nahradit\n"
      "  parametrem typu ukazatel na bázový typ.\n"
      "- Tyto prototypy funkcí jsou prakticky totožné.\n"
      "\n"
      KOD(
      "  void zpracujPole1(int pole[], int delka);\n"
      "  void zpracujPole2(int * pole, int delka);\n"
      )
      "\n"
      "- Za první parametr jde v obou případech dosadit jakkoli alokované\n"
      "  jednorozměrné pole celých čísel (tj. lokální i dynamicky alokované).\n"
      "- Uvnitř těl obou funkcí se s parametrem pole může pracovat naprosto\n"
      "  stejným způsobem.\n"
      "- S oběma funkcemi se zachází stejně i při volání.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Parametr typu pole znaků s textovým řetězcem")) return;
  if (pokracovat())
  {
    printh(
      "- Funkce zpracovávající textové řetězce často místo parametru " KK("char text[]") "\n"
      "  používají jednodušší zápis " KK("char * text") ".\n"
      "- Protože řetězec končí znakem " KK("\'\\0\'") ", často není potřeba použít parametr\n"
      "  s délkou pole. -> Funkce si konec sama najde a zastaví se tam.\n"
      "\n"
      KOD(
      "  int pocetSlov(char * text);\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Funkce s návratovým typem pole")) return;
  if (pokracovat())
  {
    printh(
      "- Takovou funkci NELZE vyrobit! Typ pole nemůže být návratovým typem.\n"
      "- Důvodem je, že výsledek takové funkce by nešel nikam přiřadit.\n"
      "\n"
      SEKCE("Návratový typ funkce typu ukazatel (na první prvek pole)")
      "- Toto má smysl, když funkce vyrábí nové pole dynamicky na hromadě (heap).\n"
      "- Vrátit ukazatel na lokální pole je nesmysl, protože takové pole leží\n"
      "  na zásobníku a při ukončení funkce zanikne. Vracel by se tedy ukazatel\n"
      "  na nealokovanou paměť!\n"
      "\n"
      KOD(
      "  int* vyrobPole(int delka)\n"
      "  {\n"
      "    return malloc(delka*sizeof(int));\n"
      "  }\n"
      "\n"
      "  // Tohle jde přeložit, ale použití povede k havárii.\n"
      "  int* vyrobPole(int delka)\n"
      "  {\n"
      "    int lokalniPole[delka];\n"
      "    return lokalniPole;\n"
      "  }\n"
      "\n"
      "  // Tohle je syntaktická chyba\n"
      "  int vyrobPole(int delka)[];\n"
      )
      "\n"
    );
  }

  konecKapitoly(__func__);
}


////////////////////////////////////////////////////////////////////////////////
void poleAPodprogramy2(void)
{
  zacatekKapitoly("Vícerozměrná pole a podprogramy");

  printh(
    "- Způsoby deklarace parametrů se liší podle způsobu výroby pole.\n"
    "- S ohledem na to je praktičtější zabalit vícerozměrné pole do struktury\n"
    "  spolu s rozměry a pracovat s ní jako s abstraktním datovým typem.\n"
    "\n"
  );

  if (navratNeboSekce("Parametr typu vícerozměrné pole s konstantními rozměry")) return;
  if (pokracovat())
  {
    printh(
      "- Rozměr první dimenze lze vynechat (stejně jako u jednorozměrného pole).\n"
      "- Rozměry dalších dimenzí se vynechat NESMÍ! Překladač je potřebuje pro\n"
      "  správný výpočet pozice prvku při indexování.\n"
      "- Je praktické předávat VŠECHNY rozměry pomocí parametrů. Funkce pak\n"
      "  může pracovat s podmnožinou prvků, když je to potřeba.\n"
      "\n"
      KOD(
      "  // Rozměr druhé dimenze nelze vynechat.\n"
      "  void zorejPole1(int lan[][SLOUPCU], int radku, int sloupcu);\n"
      )
      "\n"
      "- Jako argument jde dosadit jen pole vyrobené s konstantními rozměry.\n"
      "  Nejde takto předat ani dynamické pole, ani pole s proměnnými rozměry.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Parametr typu vícerozměrné pole s proměnnými rozměry")) return;
  if (pokracovat())
  {
    printh(
      "- Pole vyrobená na zásobníku s rozměry danými pomocí proměnných (od C99)\n"
      "  se musí předávat speciálním způsobem.\n"
      "- Jako první se musí předat parametry s rozměry a ty se pak použijí\n"
      "  v dimenzích parametru typu pole.\n"
      "- Toto má smysl jen při bastlení. Pro robustní kód je lepší dynamicky\n"
      "  alokované pole.\n"
      "\n"
      KOD(
      "  // Parametry radku a sloupcu jsou použity i tady || a tady ||\n"
      "  //                                               \\/        \\/\n"
      "  void zorejPole2(int radku, int sloupcu, int lan[radku][sloupcu]);\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Parametr typu vícerozměrné dynamické pole")) return;
  if (pokracovat())
  {
    printh(
      "- Typově jde o vícenásobný ukazatel.\n"
      "- Pole MUSÍ být správně zkonstruováno - viz další kapitola.\n"
      "- Rozměry všech dimenzí je nutné předat v samostatných parametrech.\n"
      "\n"
      KOD(
      "  void zorejPole3(int** lan, int radku, int sloupcu);\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Návratový typ funkce")) return;
  if (pokracovat())
  {
    printh(
      "- Typ pole nelze použít jako návratový typ funkce. Výsledek by nešlo\n"
      "  nikam přiřadit, viz předchozí kapitola.\n"
      "- Návratovým typem může být ukazatel (i vícenásobný). Pak funkce může\n"
      "  vracet ukazatel na pole dynamicky alokované na hromadě (heap).\n"
      "\n"
    );
  }

  konecKapitoly(__func__);
}



////////////////////////////////////////////////////////////////////////////////

void dynamickaPole(void)
{
  zacatekKapitoly("Dynamicky alokovaná pole");

  printh(
    "- Pole se alokuje dynamicky na hromadě (heap) pomocí funkce " KK("malloc") ".\n"
    "- Funkce " KK("malloc") " vrací ukazatel na alokované pole.\n"
    "- Využívá se příbuznosti typů pole a ukazatel.\n"
    "- Ukazatel jde indexovat, proto se s takto vyrobeným polem dá zacházet\n"
    "  stejně jako s polem.\n"
    "- Aby to fungovalo i u vícerozměrných polí, musí se vyrobit speciálním\n"
    "  postupem (viz dále).\n"
    "\n"
  );

  if (navratNeboSekce("Rozdíly proti použití typu pole")) return;
  if (pokracovat())
  {
    printh(
      "- Proměnná se deklaruje jako ukazatel na bázový typ.\n"
      "- Proměnnou je třeba nejprve inicializovat, jinak jde o neinicializovaný\n"
      "  ukazatel! Viz dále.\n"
      "\n"
      KOD(
      "  <bázový typ> * pole; // přesněji je to ukazatel na zatím neexistující pole\n"
      )
      "\n"
      "- Po skončení práce s ukazatelem, je nutné uvolnit paměť pomocí funkce " KK("free")".\n"
      "- Ukazatel na pole jde přiřadit do jiné proměnné typu ukazatel.\n"
      "  Nedojde však ke kopii pole. Pořád půjde jen o kopii ukazatelů.\n"
      "- Ukazatel na dynamicky alokované pole jde vracet jako funkční hodnotu funkce.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Jednorozměrné dynamické pole")) return;
  if (pokracovat())
  {
// Vypínám hlídání této chyby, protože ji chci demonstrovat.
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
    int * poleA = malloc(3 * sizeof(int)); // rychlejší, ale neinicializované
    int * poleB = calloc(3, sizeof(int));  // inicializuje prvky na nulu

    if (poleA == NULL || poleB == NULL)
    {
      printf("Neni dost pameti pro alokaci pole!\n");
      return;
    }

    printh(
      "- Pro alokaci se používá funkce " KK("malloc") ":\n"
      "  - Má 1 parametr: počet alokovaných bajtů.\n"
      "  - Počet bajtů se spočítá jako " KK("délka*sizeof(<bázový typ>)") ".\n"
      "  - Vrací ukazatel na začátek alokovaného pole nebo " KK("NULL") ".\n"
      "  - Prvky pole jsou neinicializované.\n"
      "\n"
      "- Lze použít i funkci " KK("calloc") ":\n"
      "  - Má 2 parametry: počet prvků a velikost jednoho prvku v bajtech.\n"
      "  - Velikost jednoho prvku se vždy počítá jako sizeof(bázový typ).\n"
      "  - Vrací ukazatel na začátek alokovaného pole nebo " KK("NULL") ".\n"
      "  - Prvky pole nastaví na hodnotu nula.\n"
      "  - Pomalejší než " KK("malloc") ".\n"
      "\n"
      KOD(
      "  int * poleA = malloc(3 * sizeof(int)); // rychlejší, ale neinicializované\n"
      "  int * poleB = calloc(3, sizeof(int));  // inicializuje prvky na nulu\n"
      )
      "\n"
      "- Prvky poleA mohou také mít hodnotu nula, ale nedá se na to spoléhat.\n"
      "\n"
      "  => poleA: |%d %d %d| (překladač hlásí varování při použití poleA)\n"
      "  => poleB: |%d %d %d|\n"
      "\n"
      "- Po použití je pole třeba uvolnit pomocí funkce " KK("free") ".\n"
      "\n"
      KOD(
      "  free(poleA);\n"
      "  free(poleB);\n"
      )
      "\n"
      , poleA[0], poleA[1], poleA[2] // překladač hlásí varování
      , poleB[0], poleB[1], poleB[2]
    );

    free(poleA);
    free(poleB);
#pragma GCC diagnostic pop
  }

  konecKapitoly(__func__);
}



////////////////////////////////////////////////////////////////////////////////


typedef struct
{
  int delka;   // délka pole
  int * prvek; // pole prvků
} Tpole;

/** Dynamicky alokuje nové pole zastřešené strukturou Tpole.
 *
 * \param delka int Požadovaná délka pole.
 * \return Tpole* Vrací ukazatel na alokovanou strukturu s polem, nebo NULL.
 */
Tpole * novePole(int delka)
{
  Tpole * a = malloc(sizeof(Tpole)); // alokuje strukturu
  if (a != NULL) // když se alokace nepovede, vracím NULL
  {
    a->prvek = malloc(delka * sizeof(int)); // alokuje prvky pole
    if (a->prvek == NULL)
    {
      free(a);     // alokace se nepovedla, uvolním strukturu vracím NULL
      return NULL;
    }
    a->delka = delka;
  }

  return a;
}

/** Uvolní dynamicky alokované pole zabalené ve struktuře Tpole.
 *
 * \param pole Tpole* Ukazatel na alokovanou strukturu.
 * \return void
 */
void uvolniPole(Tpole *pole)
{
  free(pole->prvek);
  free(pole);
}

void dynamickaPoleADT1D(void)
{
  zacatekKapitoly("Abstraktní datový typ 1D pole");

  printh(
    "- S polem je potřeba do funkcí předávat i délku. => Pro lepší přehlednost\n"
    "  lze zabalit ukazatel na pole i délku do struktury.\n"
    "\n"
    KOD(
    "  typedef struct\n"
    "  {\n"
    "    int delka;   // délka pole\n"
    "    int * prvek; // pole prvků\n"
    "  } Tpole;\n"
    )
    "\n"
  );

  if (navratNeboSekce("Základní operace")) return;
  if (pokracovat())
  {
    printh(
      "- Pro práci s tímto ADT je potřeba udělat funkce pro alokaci a uvolnění\n"
      "  (kód viz zdrojový text).\n"
      "- Funkce " KK("novePole") " dynamicky alokuje strukturu a uvnitř ní pole zadané délky.\n"
      "- Funkce " KK("uvolniPole") " uvolní jak pole, tak obalující strukturu.\n"
      "\n"
      KOD(
      "  Tpole * novePole(int delka);\n"
      "  void uvolniPole(Tpole *pole);\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Práce s ADT 1D pole")) return;
  if (pokracovat())
  {
    Tpole * p = novePole(3);
    if (p == NULL) return; // není dost paměti

    for (int i = 0; i < p->delka; ++i)
    {
      p->prvek[i] = i;
    }

    printh(
      "- Pole je složkou uvnitř proměnné typu struktura.\n"
      "- Funkce " KK("novePole") " vrací ukazatel na tuto proměnnou, proto šipkový operátor.\n"
      "- Složka " KK("prvek") " je ukazatel na pole, indexuje se tedy jako pole.\n"
      "\n"
      KOD(
      "  Tpole * p = novePole(3);\n"
      "  if (p == NULL) return; // není dost paměti\n"
      "  \n"
      "  // inicializace průchodem všemi prvky\n"
      "  for (int i = 0; i < p->delka; ++i)\n"
      "  {\n"
      "    p->prvek[i] = i; // nebo jiná inicializace prvků\n"
      "  }\n"
      )
      "\n"
      "- Proměnná " KK("p") " teď ukazuje na strukturu obsahující jak pole, tak jeho délku.\n"
      "\n"
      "  => p->delka: %d\n"
      "  => p->prvek: |%d %d %d|\n"
      "\n"
      "- Po ukončení práce s polem je potřeba uvolnit paměť.\n"
      "\n"
      KOD(
      "  uvolniPole(p);\n"
      )
      "\n"
      , p->delka
      , p->prvek[0], p->prvek[1], p->prvek[2]
    );

    uvolniPole(p);
  }

  konecKapitoly(__func__);
}



////////////////////////////////////////////////////////////////////////////////

typedef struct
{
  int radku, sloupcu; // rozměry dimenzí
  int ** prvek;       // ukazatel na matici prvků
} TMatice;

/** Uvolní paměť alokovanou pro matici. Musí se to dělat ve správném pořadí.
 * Nejprve se uvolní jednotlivé řádky, pak pomocné pole ukazatelů na řádky
 * a teprve nakonec se uvolní obalující struktura.
 */
void uvolniMatici(TMatice *m)
{
  for (int r = 0; r < m->radku; ++r)
  {
    free(m->prvek[r]); // zruší řádek
  }
  free(m->prvek); // zruší pomocné pole ukazatelů na řádky

  free(m);  // zruší obalující strukturu
}

/** Alokuje strukturu obalující matici (2d pole) a rozměry.
 *
 * Matice (2d pole) se deklaruje jako dvojitý ukazatel. Tvoří ji pole ukazatelů
 * na řádky a pak jednotlivě alokované řádky. Tato pole je potřeba alokovat
 * ve správném pořadí.
 *
 * Funkce vrací NULL, když se během alokování něco nepovede.
 */
TMatice * novaMatice(int radku, int sloupcu)
{
  if (radku <= 0 || sloupcu <= 0)
    return NULL;

  TMatice * m = malloc(sizeof(TMatice)); // alokuje strukturu
  if (m != NULL) // když se alokace nepovede, vracím NULL
  {
    m->prvek = malloc(radku * sizeof(int*)); // typ int* => alokuji pole ukazatelů na řádky
    if (m->prvek == NULL)
    {
      free(m);     // alokace se nepovedla, uvolním strukturu vracím NULL
      return NULL;
    }

    for (int r = 0; r < radku; ++r)
    {
      m->prvek[r] = malloc(sloupcu * sizeof(int)); // typ int => řádek je jednoduché pole
      if (m->prvek[r] == NULL)
      {
        m->radku = r;     // počet skutečně alokovaných řádků
        uvolniMatici(m);  // alokace se nepovedla, uvolním strukturu vracím NULL
        return NULL;
      }
    }

    m->radku = radku;
    m->sloupcu = sloupcu;
  }

  return m;
}


void dynamickaPoleADT2D(void)
{
  zacatekKapitoly("Abstraktní datový typ 2D pole (matice)");

  printh(
    "- S vícerozměrným polem je potřeba do funkcí předávat i rozměry dimenzí.\n"
    "  => Pro lepší přehlednost lze zabalit ukazatel na pole i jednotlivé\n"
    "  rozměry dimenzí do struktury.\n"
    "\n"
    KOD(
    "  typedef struct\n"
    "  {\n"
    "    int radku, sloupcu; // rozměry dimenzí\n"
    "    int ** prvek;       // ukazatel na matici prvků\n"
    "  } TMatice;\n"
    )
    "\n"
    "- Samotné pole je teď deklarováno jako dvojitý ukazatel.\n"
    "- Matici tvoří pomocné pole ukazatelů na řádky a samostatně alokované\n"
    "  řádky.\n"
    "- Musí se to alokovat postupně pomocí cyklu (viz zdrojový kód).\n"
    "\n"
    "- Při uvolňování paměti se musí postupovat opačně - nejprve se uvolní\n"
    "  jednotlivé řádky, pak pomocné pole a nakonec obalující struktura.\n"
    "\n"
    "- U vícerozměrných polí se postupuje analogicky.\n"
    "  Poznámka: Použití více než 2 dimenzí je obvykle známka nevhodného návrhu.\n"
    "\n"
  );

  if (navratNeboSekce("Základní operace")) return;
  if (pokracovat())
  {
    printh(
      "- Pro práci s tímto ADT je potřeba udělat funkce pro alokaci a uvolnění\n"
      "  (kód viz zdrojový text).\n"
      "- Funkce " KK("novaMatice") " dynamicky alokuje strukturu a uvnitř ní pole zadaných\n"
      "  dimenzí.\n"
      "- Funkce " KK("uvolniMatici") " uvolní jak pole, tak obalující strukturu.\n"
      "\n"
      KOD(
      "  TMatice * novaMatice(int radku, int sloupcu);\n"
      "  void uvolniMatici(TMatice *m);\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Práce s maticí (ADT 2D pole)")) return;
  if (pokracovat())
  {
    TMatice * m = novaMatice(3, 5);
    if (m == NULL) return; // není dost paměti

    for (int r = 0; r < m->radku; ++r)
      for (int s = 0; s < m->sloupcu; ++s)
      {
        m->prvek[r][s] = r*m->sloupcu + s;
      }

    printh(
      "- Pole je složkou uvnitř proměnné typu struktura.\n"
      "- Funkce " KK("novaMatice") " vrací ukazatel na tuto proměnnou, proto šipkový operátor.\n"
      "- Složka prvek je ukazatel na pole, indexuje se tedy jako pole.\n"
      "\n"
      KOD(
      "  TMatice * m = novaMatice(3, 5);\n"
      "  if (m == NULL) return; // není dost paměti\n"
      "\n"
      "  // inicializace průchodem všemi prvky\n"
      "  for (int r = 0; r < m->radku; ++r)\n"
      "    for (int s = 0; s < m->sloupcu; ++s)\n"
      "    {\n"
      "      m->prvek[r][s] = r*m->sloupcu + s;  // nebo jiná inicializace\n"
      "    }\n"
      )
      "\n"
      "- Proměnná m teď ukazuje na strukturu obsahující jak pole, tak jeho dimenze.\n"
      "\n"
      "  => m->radku: %d\n"
      "  => m->sloupcu: %d\n"
      "  => m->prvek: |%2d %2d %2d %2d %2d|\n"
      "               |%2d %2d %2d %2d %2d|\n"
      "               |%2d %2d %2d %2d %2d|\n"
      "\n"
      "- Po ukončení práce s polem je potřeba uvolnit paměť.\n"
      "\n"
      KOD(
      "  uvolniMatici(m);\n"
      )
      "\n"
      , m->radku
      , m->sloupcu
      , m->prvek[0][0], m->prvek[0][1], m->prvek[0][2], m->prvek[0][3], m->prvek[0][4]
      , m->prvek[1][0], m->prvek[1][1], m->prvek[1][2], m->prvek[1][3], m->prvek[1][4]
      , m->prvek[2][0], m->prvek[2][1], m->prvek[2][2], m->prvek[2][3], m->prvek[2][4]
    );

    uvolniMatici(m);
  }

  konecKapitoly(__func__);
}



////////////////////////////////////////////////////////////////////////////////
void realokacePole(void)
{
  zacatekKapitoly("Realokace pole");

  printh(
    "- Velikost dynamicky alokovaného pole lze změnit pomocí funkce " KK("realloc") ".\n"
    "\n"
  );

  if (navratNeboSekce("Jak funguje realokování")) return;
  if (pokracovat())
  {
    printh(
      "- Funkce " KK("realloc") " mění velikost alokované paměti.\n"
      "\n"
      "- Je-li nová velikost menší než původní:\n"
      "  => uvolní přebytečnou část pole,\n"
      "  => vrací se nezměněný ukazatel na začátek pole.\n"
      "\n"
      "- Při zvětšování mohou nastat 2 možnosti:\n"
      "  1. Za polem je dost volného místa.\n"
      "     => To se alokuje a sloučí s původním polem.\n"
      "     => Vrací se nezměněný ukazatel na začátek pole.\n"
      "  2. Za polem není dost volného místa.\n"
      "     => Alokuje se nové pole s novou velikostí jinde.\n"
      "     => Prvky z původního pole se zkopírují do nového pole.\n"
      "     => Původní pole se uvolní.\n"
      "     => Vrací se ukazatel na nově alokované pole.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Použití realloc")) return;
  if (pokracovat())
  {
    int * pole = malloc(2*sizeof(int)); // pole o velikosti 2

    int *pom = realloc(pole, 10*sizeof(int));  // nová velikost má být 10
    if (pom != NULL) // pokud došlo k realokaci, pole může ležet jinde
    {
      pole = pom;
    }

    // tady se zase pracuje s proměnnou pole

    free(pole);

    printh(
      "\n"
      KOD(
      "  int * pole = malloc(2*sizeof(int)); // pole o velikosti 2\n"
      "  \n"
      "  // ... někde zjistím, že původní velikost nestačí\n"
      "  \n"
      "  int *pom = realloc(pole, 10*sizeof(int));  // nová velikost má být 10\n"
      "  if (pom != NULL) // pokud došlo k realokaci, pole může ležet jinde\n"
      "  {\n"
      "    pole = pom;\n"
      "  }\n"
      "  // else ... pokud pom==NULL, realokace neproběhla => musím se s tím vyrovnat\n"
      "  \n"
      "  // tady se zase pracuje s proměnnou pole\n"
      "  \n"
      "  free(pole);\n"
      )
      "\n"
      "- První parametr funkce " KK("realloc") " je ukazatel na původní pole.\n"
      "  Je-li sem dosazena hodnota " KK("NULL") ", chová se to jako " KK("malloc") ".\n"
      "- Druhý parametr je nová velikost pole (v bajtech).\n"
      "\n"
    );
  }

  if (navratNeboSekce("Chybné použití realloc")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  pole = realloc(pole, 10*sizeof(int)); // proměnná pole je tu použita dvakrát!\n"
      )
      "\n"
      "- Toto je častá začátečnická chyba.\n"
      "- Dojde-li k chybě a " KK("realloc") " vrátí " KK("NULL") ", dojde ke ztrátě ukazatele na\n"
      "  původní pole => paměťový únik (memory leak).\n"
      "- Pokud " KK("realloc") " vrací " KK("NULL") " (kvůli nedostatku paměti), nemění původní pole.\n"
      "  Tady se ale hodnota " KK("NULL") " do proměnné pole zapíše a tím se ztratí vazba\n"
      "  na původní alokovanou paměť.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Strategie použití nafukovacího pole")) return;
  if (pokracovat())
  {
    printh(
      "- Typicky se používá pro načtení prvků ze souboru předem neznámé délky.\n"
      "- POZOR! Nezvětšovat o 1 prvek!\n"
      "  => Každé volání " KK("realloc") " může znamenat kopii pole.\n"
      "\n"
      PODSEKCE("Strategie konstantních přírůstků")
      "- Když mi v poli dojde místo, zvětším jej o konstantní velikost,\n"
      "  například 64 prvků.\n"
      "- Hodí se, když je předpokládaná velikost dat relativně malá.\n"
      "\n"
      PODSEKCE("Exponenciální strategie")
      "- Když mi v poli dojde místo, zvětším velikost na dvojnásobek.\n"
      "- Hodí se, když je předpokládaná velikost dat velká.\n"
      "\n"
      "- Na konci čtení lze přebytečně alokovaný zbytek pole uvolnit zase pomocí\n"
      "  funkce " KK("realloc") ".\n"
      "\n"
    );
  }

  konecKapitoly(__func__);
}

