/* Encyklopedie jazyka C
 *
 * Modul s menu
 *
 * Tento hlavičkový soubor obsahuje makra a hlavičky podprogramů pro realizaci
 * obecného nabídkového menu. Jejich implementace se nachází ve zdrojovém
 * souboru modulu.
 *
 * Tento hlavičkový soubor je potřeba do zdrojového textu vložit vždy jako
 * první, ještě před vložením systémových hlavičkových souborů.
 *
 * 2023-07-31-22.52
 */

/* Copyright (c) David Martinek, 2023
 * david.martinek.78 (zavináč) gmail.com
 *
 * This work is licensed under a Creative Commons
 * Attribution-NonCommercial-ShareAlike 4.0 International License.
 * (CC BY-NC-SA 4.0)
 * https://creativecommons.org/licenses/by-nc-sa/4.0/
 *
 * Toto dílo je licencováno pole licence Creative Commons
 * Uveďte původ-Neužívejte dílo komerčně-Zachovejte licenci 4.0 Mezinárodní
 * (CC BY-NC-SA 4.0)
 * https://creativecommons.org/licenses/by-nc-sa/4.0/deed.cs
 */

#ifndef MENU_H_INCLUDED
#define MENU_H_INCLUDED

#include "pomocny-modul.h"
#include <stdbool.h>

/** Typ ukazatel na funkci s hlavičkou tvaru void fce(void).
 * V takovém tvaru musí být obslužné funkce jednotlivých položek menu.
 */
typedef void (* TFobsluha)(void);


/** Abstraktní datový typ pro realizaci menu.
 * Položky jsou skryté.
 * Jde používat pouze typ Tmenu* prostřednictvím veřejných funkcí tohoto modulu.
 */
typedef struct _menu Tmenu;


/** Vytvoří nové menu se zadanou hlavičkou.
 * \param hlavicka const char* Neměnný textový řetězec reprezentující hlavičku menu.
 * \param akcePredVolbou TFobsluha Funkce, která se zavolá po vykreslení menu.
 * \return Tmenu* Vrací ukazatel na menu nebo NULL.
 */
Tmenu * menuVytvor(const char *hlavicka, TFobsluha akcePredVolbou);


/** Uvolní dynamicky alokovanou paměť pro menu a všechny jeho položky.
 * \param menu Tmenu* Ukazatel na vyrobené menu.
 */
void menuZrus(Tmenu * menu);


/** Vytvoří a přidá další položku menu do seznamu.
 *
 * Pokud je aktivační znak nastaven na hodnotu 0, jde o položku menu, která
 * nemá přiřazenou žádnou akci. Může jít o nadpis, poznámku a podobně.
 * Předpokládá se, že jako za parametr akce bude dosazena hodnota NULL.
 *
 * \param menu Tmenu* Ukazatel na vytvořené menu.
 * \param aktivator char Znak, který aktivuje tuto nabídku.
 * \param nabidka const char* Text nabídky.
 * \param akce TFobsluha Funkce, která se vykoná při výběru této nabídky.
 * \return bool Vrací true, když se položku povede přidat, jinak false.
 */
bool menuPridej(Tmenu * menu, char aktivator, const char * nabidka, TFobsluha akce);


/** Realizuje výpis menu, čekání na výběr varianty a volání akce registrované k této variantě.
 *
 * Pokud uživatel zadá volbu Q nebo Enter, vypíše hlášení o konci programu.
 * Vždy vrací velké písmeno.
 * Pokud je zvoleno jiné písmeno, než se nachází v položkách menu, tiskne
 * hlášení, že to má uživatel zkusit znovu.
 *
 * Nerealizuje cyklus opakovaného výpisu menu. Menu vypíše pouze jednou, počká
 * na volbu a pokud se daná volba v menu nachází, zavolá zaregistrovanou akci.
 *
 * \param menu Tmenu* Ukazatel na inicializované menu.
 * \return int Vrací písmeno volby.
 */
int menuVolba(Tmenu *menu);


/** Realizuje podmenu pro nastavení rychlosti vyprávění.
 *
 * Jde přepínat mezi statickým a vyprávěcím módem aplikace.
 *
 * Vyprávěcí mód funguje tak, že řádkem běží kurzor a zanechává za sebou text.
 * Jde nastavit tři různé rychlosti tohoto módu. Čtenáři by měl vyprávěcí mód
 * pomáhat udržovat pozornost.
 *
 * Pokud čtenáři vyprávěcí mód vadí, může si zapnout mód statický, který zobrací
 * celý blok textu najednou bez čekání.
 */
void nastavRychlostVypraveni(void);

#endif // MENU_H_INCLUDED
