/* Encyklopedie jazyka C
 *
 * Modul s pomocnými podprogramy a makry.
 *
 * Tento hlavičkový soubor obsahuje makra a hlavičky pomocných podprogramů
 * pro práci s terminálovým výstupem.
 * Jejich implementace se nachází ve zdrojovém souboru modulu.
 *
 * Tento hlavičkový soubor je potřeba do zdrojového textu vložit vždy jako
 * první, ještě před vložením systémových hlavičkových souborů.
 *
 * 2023-08-20-11.09
 * 2023-08-10-09.41
 * 2023-07-31-23.07
 */

/* Copyright (c) David Martinek, 2021--2023
 * david.martinek.78 (zavináč) gmail.com
 *
 * This work is licensed under a Creative Commons
 * Attribution-NonCommercial-ShareAlike 4.0 International License.
 * (CC BY-NC-SA 4.0)
 * https://creativecommons.org/licenses/by-nc-sa/4.0/
 *
 * Toto dílo je licencováno pole licence Creative Commons
 * Uveďte původ-Neužívejte dílo komerčně-Zachovejte licenci 4.0 Mezinárodní
 * (CC BY-NC-SA 4.0)
 * https://creativecommons.org/licenses/by-nc-sa/4.0/deed.cs
 */

// Toto zabraňuje vícenásobnému vložení hlavičkového souboru do zdrojového textu.
#ifndef _POMOCNY_MODUL_H_
#define _POMOCNY_MODUL_H_

#include <stdbool.h>
#include <stdlib.h>

// Tohle kouzlo je podmíněný překlad, který detekuje, zda se překládá ve Windows.
#if defined WIN32 || defined _WIN32 || defined WIN64 || defined _WIN64

  // Tohle zaříkání je nutné použít v překladači MINGW ve Windows, protože
  // překladač běžně používá céčkovou knihovnu od MS, která nemá správně
  // implementováno rozšíření C99 a novější. Tento přepínač donutí překladač
  // používat knihovnu šířenou s MINGW. Bez tohoto woodoo ve Windows nefungují
  // například některé formátovací značky příkazu printf (%zd, %zu, %hhd,
  // %lld aj.).
  // Aby tohle kouzlo zafungovalo, je potřeba tento define mít ve zdrojovém
  // textu před systémovými #include.
  // Také je dobré si sundat ponožku, ukázat bosou nohou do čtyř světových
  // stran a přát si brzký a potupný zánik firmy MS.
  #define __USE_MINGW_ANSI_STDIO 1
#endif




// ANSI kódy pro barevný výstup na terminálu. Funguje to tak, že když se na
// terminál vytiskne příslušný ANSI kód, terminál jej neinterpretuje jako
// text, ale jako řídící kód, který jej přinutí změnit barvu. Barevná část
// textu začíná kódem barvy a musí končit resetovacím kódem.

/**
 * Přepne terminál do módu, ve kterém funguje barevný výstup.
 * Tato funkce je potřeba kvůli terminálu ve Windows.
 */
void povolBarevnyVystup(void);

// Resetovací kód
#define CEND "\033[0m"

// kódy barev písma
#define BLACK   "\033[0;30m"
#define RED     "\033[0;31m"
#define GREEN   "\033[0;32m"
#define YELLOW  "\033[0;33m"
#define BLUE    "\033[0;34m"
#define PURPLE  "\033[0;35m"
#define CYAN    "\033[0;36m"
#define WHITE   "\033[0;37m"

// Bold - tučné písmo
#define BBLACK  "\033[1;30m"
#define BRED    "\033[1;31m"
#define BGREEN  "\033[1;32m"
#define BYELLOW "\033[1;33m"
#define BBLUE   "\033[1;34m"
#define BPURPLE "\033[1;35m"
#define BCYAN   "\033[1;36m"
#define BWHITE  "\033[1;37m"


// Underline - podtržené písmo
#define UBLACK   "\033[4;30m"
#define URED     "\033[4;31m"
#define UGREEN   "\033[4;32m"
#define UYELLOW  "\033[4;33m"
#define UBLUE    "\033[4;34m"
#define UPURPLE  "\033[4;35m"
#define UCYAN    "\033[4;36m"
#define UWHITE   "\033[4;37m"

// Background - barva pozadí - funguje jen někde
#define BGBLACK  "\033[40m"
#define BGRED    "\033[41m"
#define BGGREEN  "\033[42m"
#define BGYELLOW "\033[43m"
#define BGBLUE   "\033[44m"
#define BGPURPLE "\033[45m"
#define BGCYAN   "\033[46m"
#define BGWHITE  "\033[47m"

// High Intensity - funguje jen někde
#define IBLACK    "\033[0;90m"
#define IRED      "\033[0;91m"
#define IGREEN    "\033[0;92m"
#define IYELLOW   "\033[0;93m"
#define IBLUE     "\033[0;94m"
#define IPURPLE   "\033[0;95m"
#define ICYAN     "\033[0;96m"
#define IWHITE    "\033[0;97m"

// Bold High Intensity - funguje jen někde
#define BIBLACK   "\033[1;90m"
#define BIRED     "\033[1;91m"
#define BIGREEN   "\033[1;92m"
#define BIYELLOW  "\033[1;93m"
#define BIBLUE    "\033[1;94m"
#define BIPURPLE  "\033[1;95m"
#define BICYAN    "\033[1;96m"
#define BIWHITE   "\033[1;97m"

// Barevné schéma
#define _KOD BPURPLE
#define _NADPIS BIBLUE
#define _SEKCE BBLUE UBLUE
#define _PODSEKCE BBLUE
#define _VOLBA RED

// Makra pro sázení textu.
// Parametrem je textový literál, výsledkem textový literál doplněný
// o barvící značky.
#define NADPIS(text) _NADPIS text CEND
#define SEKCE(text) _SEKCE text CEND "\n"
#define PODSEKCE(text) _PODSEKCE text CEND "\n"
#define VOLBA(text) _VOLBA text CEND
#define BB(text) BBLUE text CEND

// Původně zamýšleno jako víceřádková varianta
#define KOD(text) _KOD text CEND
// Původně zamýšleno jako varianta pro jedno slovo
#define KK(text) _KOD text CEND

/* Implementační poznámka: Nepoužívám stringifikaci parametrů protože to v tomto
případě naráží na potíže. (1) Kvůli textům obsahujícím čárky by makra musela
být variadická. (2) Veškeré přebytečné mezery se vždy smrsknou na jedinou
mezeru, takže nejde dělat případné formátování mezerami. (3) Parametry nesmí
obsahovat komentáře, což někdy potřebuji. Detaily viz kapitola Preprocesor.
*/

/** Vyčistí konzoli. */
void vycistiKonzoli(void);

/** Podprogram ze standardního vstupu přečte všechny znaky až do konce řádku.
 * Vyčistí tak vstup pro zadávání další hodnoty.
 *
 * Dá se také použít pro čekání, až uživatel stiskne Enter.
 */
void pozerKonecRadku(void);

/** Vrátí první nebílý znak zapsaný na řádku.
 * Pak požere všechny ostatní znaky až do konce řádku.
 */
int prvniPismenoNaRadku();

/** Pokud uživatel odpoví Q, vrací true, jinak tiskne hlavičku další sekce. */
bool navratNeboSekce(char * nasledujiciSekce);

bool pokracovat(void);

/** Jednotně zobrazí nadpis kapitoly. */
void zacatekKapitoly(const char *nazev);

/** Počká na stisk klávesy Enter na konci kapitoly. */
void konecKapitoly(const char * jmenoFunkce);







/** Nastaví mód tisku pomocí printh.
 * \param vypravet bool Pokud je nastaven na true, nastaví se vyprávěcí mód, jinak statický.
 */
void nastavVypraveciMod(bool vypravet);

/** Přepne mód tisku pomocí printh z vyprávěcího na statický nebo naopak. */
void prepniVypraveciMod(void);

/** Vrací true, když je nastaven vyprávěcí mód pro printh. */
bool jeVypraveciMod(void);

/** Zobrazí, jaký je právě nastavený mód tisku. */
void zobrazMod(void);

/** Rychlosti vyprávění - je to zpoždění mezi tiskem dvou písmen v milisekundách. */
enum eRychlost { POMALA=30, STREDNI=17, RYCHLA=10 };

/** Přepne rychlost vyprávění. Pro nastavení rychlosti jsou připraveny konstanty eRychlost. */
void prepniRychlostVypraveni(int rychlost);


/** Funguje jako printf, ale převede formátovací řetězec do CP852. */
int printg(const char * format, ...);

/** Funguje jako sprintf, ale převede formátovací řetězec do CP852. */
int sprintg(char * str, const char * format, ...);

/**
 * Funguje jako printg, ale používá vyprávěcí mód, pokud je zapnut.
 *
 * Vyprávěcí mód funguje tak, že řádkem běží kurzor a zanechává za sebou text.
 * Jde nastavit tři různé rychlosti tohoto módu. Čtenáři by měl vyprávěcí mód
 * pomáhat udržovat pozornost.
 *
 * Mezi vyprávěcím a statickým módem přepíná funkce nastavVypraveciMod.
 *
 * Rychlost vyprávěcího módu přepíná funkce prepniRychlostVypraveni.
 */
int printh(const char * format, ...);


#endif // _POMOCNY_MODUL_H_
